'
' dfsutil.vbs
' Database Utility
'
' Copyright (c) 2012-2014 SAS Institute Inc.
'

' constants from MSDN
Const ForReading = 1
Const ForWriting = 2

' usage strings
usage = "usage: dfsutil <command> [<options>]"
version = "SAS Federation Server Database Utility version 4.2"

' command line or GUI? 
If InStr(UCase(WScript.FullName), "CSCRIPT") Then
    isGUI = false
Else
    isGUI = true
End If

' return true if path is absolute
Function isAbs(path)
    isAbs = false
    c1 = Mid(path, 1, 1)
    If c1 = "\" Or c1 = "/" Then
        ' UNC path or abs path on current volume
        isAbs = true
    Else 
        c2 = Mid(path, 2, 2)
        If c2 = ":\" Then
            ' C:\foo\bar
            isAbs = true
        End If
    End If
End Function

' find root of installation 
Set fso = CreateObject("Scripting.FileSystemObject")
bindir = fso.GetFile(WScript.ScriptFullName).ParentFolder
home = bindir & "\..\"
etcdir = home & "etc\"


' default settings
app_name = "Federation Server Password Encoder"

' read options from XML config - dfs_serv.xml
Set xml = CreateObject("MSXML2.DOMDocument")
xml.Async = False
xml.ValidateOnParse = False
xml.load(etcdir & "dfs_serv.xml")
If xml.ParseError.ErrorCode <> 0 Then
    WScript.Echo "Unable to parse dfs_serv.xml"
    WScript.Echo xml.ParseError.Reason
    WScript.Quit 1
End If
Set xmlOpts = xml.getElementsByTagName("Option")
Set opts = CreateObject("Scripting.Dictionary")
For Each x in xmlOpts
    optName = x.getAttribute("name")
    optValue = x.text
    If Not opts.Exists(optName) Then
        opts.Add optName, optValue
    End If
    ' WScript.Echo optName & " = " & optValue
Next

' read entities from XML config - dfs_log.xml
xml.load(etcdir & "dfs_log.xml")
If xml.ParseError.ErrorCode <> 0 Then
    WScript.Echo "Unable to parse dfs_log.xml"
    WScript.Echo xml.ParseError.Reason
    WScript.Quit 1
End If
Set EntList = xml.DocType.entities
Set ents = CreateObject("Scripting.Dictionary")
For Each item in EntList
    entName = item.nodeName
    entValue = item.text
    If Not ents.Exists(entName) Then
        ents.Add entName, entValue
    End If
    ' WScript.Echo entName & " = " & entValue
Next

' Content Root
contentRoot = opts.Item("ContentRoot")
If contentRoot <> "" Then
    If Not isAbs(contentRoot) Then
        contentRoot = home & "\" & contentRoot
    End If
End If

' TranPath
tranPath = opts.Item("TranPath")
If tranPath <> "" Then
    If Not isAbs(tranPath) Then
        If contentRoot <> "" Then
            tranPath = contentRoot & "\" & tranPath
        Else
            tranPath = home & "\" & tranPath
        End If
    End If
End IF

' set up environment
Set shell = WScript.CreateObject("WScript.Shell")
Set env = shell.Environment("PROCESS")
If ents.Exists("cfg.home.install") Then
    homeInstall = ents.Item("cfg.home.install")
else
    homeInstall = home
End If
fbDir = homeInstall & "\lib\fbembed"
env("FIREBIRD") = fbDir
env("FIREBIRD_LOG") = home & "var\log"
tranPath = opts.Item("TranPath")
If tranPath = "" Then
    env("FIREBIRD_LOCK") = home & "var"
Else
    env("FIREBIRD_LOCK") = tranPath
End If

' determine database location - default: syscat.tdb
dbFile = opts.Item("Database")
If dbFile = "" Then
    ' default location
    dbFile = home & "var\syscat.tdb"
Else
    ' if path is absolute, leave alone
    If Not isAbs(dbFile) Then
        If opts.Item("TranPath") <> "" Then
            dbFile = opts.Item("TranPath") & "\" & dbFile
        Else
            If opts.Item("ContentRoot") <> "" Then
                dbFile = opts.Item("ContentRoot") & "\" & dbFile
            Else
                dbFile = home & "var\" & dbFile
            End If
        End If
    End If
End If

' process command line
If WScript.Arguments.Count < 1 Then
    WScript.Echo usage
    WScript.Echo "(Run 'dfsutil help' for a list of commands and options.)"
    WScript.Quit 1
End If
cmdName = WScript.Arguments(0)

' handle commands
If cmdName = "backup" Then
    ' backup database to provided location
    If WScript.Arguments.Count < 2 Then
        WScript.Echo "usage: dfsutil backup [-db syscat|sql_log] [<options>] <backup_path>"
        WScript.Quit 1
    End If

    ' check for options -db parameter
    If (StrComp(WScript.Arguments(1), "-db",1) = 0) Then
        dbType = WScript.Arguments(2)
        If ((StrComp(dbType,"syscat",1) <> 0) and (StrComp(dbType,"sql_log",1) <> 0)) Then
            WScript.Echo "ERROR: incorrect database option. Values can be: syscat -or- sql_log"
            WScript.Quit 1
        End If
    End If

    ' If dbType is sql_log then re-assign dbFile
    If (StrComp(dbType,"sql_log",1) = 0) Then
        dbFile = ents.Item("DFS_DBAPPENDER_DB")
        If dbFile = "" Then
            dbFile = home & "var\sql_log.tdb"
        Else
            ' if path is absolute, leave alone
            If Not isAbs(dbFile) Then
                dbFile = home & "var\" & dbFile
            End If
        End IF

    End If

    backupFile = WScript.Arguments(WScript.Arguments.Count - 1)
    '
    ' Here we need to see if the user passed in the optional: -db [syscat|sql_log]
    ' parameter, if so skip these when adding items to the extraArgs
    '
    If dbType <> "" Then
        argStart = 3
    Else
        argStart = 1
    End If

    For i = argStart to WScript.Arguments.Count - 2
        extraArgs = extraArgs & " """ & WScript.Arguments(i) & """"
    Next
    cmd = """" & fbDir & "\gbak.exe" & """ -b """ & dbFile & """ """ & backupFile & """ -user sysdba " & extraArgs
ElseIf cmdName = "restore" Then
    ' restore database from provided location
    If WScript.Arguments.Count < 2 Then
        WScript.Echo "usage: dfsutil restore [-db syscat|sql_log] [<options>] <backup_path>"
        WScript.Quit 1
    End If

    ' check for options -db parameter
    If (StrComp(WScript.Arguments(1), "-db",1) = 0) Then
        dbType = WScript.Arguments(2)
        If ((StrComp(dbType,"syscat",1) <> 0) and (StrComp(dbType,"sql_log",1) <> 0)) Then
            WScript.Echo "ERROR: incorrect database option. Values can be: syscat -or- sql_log"
            WScript.Quit 1
        End If
    End If

    ' If dbType is sql_log then re-assign dbFile
    If (StrComp(dbType,"sql_log",1) = 0) Then
        dbFile = ents.Item("DFS_DBAPPENDER_DB")
        If dbFile = "" Then
            dbFile = home & "var\sql_log.tdb"
        Else
            ' if path is absolute, leave alone
            If Not isAbs(dbFile) Then
                dbFile = home & "var\" & dbFile
            End If
        End IF

    End If

    backupFile = WScript.Arguments(WScript.Arguments.Count - 1)
    '
    ' Here we need to see if the user passed in the optional: -db [syscat|sql_log]
    ' parameter, if so skip these when adding items to the extraArgs
    '
    If dbType <> "" Then
        argStart = 3
    Else
        argStart = 1
    End If

    backupFile = WScript.Arguments(WScript.Arguments.Count - 1)
    For i = argStart to WScript.Arguments.Count - 2
        extraArgs = extraArgs & " """ & WScript.Arguments(i) & """"
    Next
    cmd = """" & fbDir & "\gbak.exe" & """ -r o """ & backupFile & """ """ & dbFile & """ -user sysdba " & extraArgs
ElseIf cmdName = "exec" Then
    ' execute FB utility directly
    util = WScript.Arguments(1)
    For i = 2 to WScript.Arguments.Count - 1
        extraArgs = extraArgs & " """ & WScript.Arguments(i) & """"
    Next
    cmd = """" & fbDir & "\" & util & extraArgs
ElseIf cmdName = "help" Then
    WScript.Echo usage
    WScript.Echo "Commands:"
    WScript.Echo "    backup     run backup utility"
    WScript.Echo "               options: -db [syscat|sql_log] (Optional)"
    WScript.Echo "                        <backup_location>"
    WScript.Echo "    restore    run restore utility"
    WScript.Echo "               options: -db [syscat|sql_log] (Optional)"
    WScript.Echo "                        <backup_location>"
    WScript.Echo "    help       display this message"
    WScript.Echo "    version    display version information"
    WScript.Quit 0
ElseIf cmdName = "version" Then
    WScript.Echo version
    WScript.Quit 0
Else
    WScript.Echo "ERROR: unknown command '" & cmdName & "'."
    WScript.Echo "(Run 'dfsutil help' for a list of commands and options.)"
    WScript.Quit 1
End If

' run command for backup/restore
WScript.Echo cmd
Set process = shell.Exec(cmd)
While Not process.StdOut.AtEndOfStream 
    line = process.StdOut.ReadLine
    WScript.StdOut.WriteLine line
Wend

